/*
 * Copyright (c) 2010-2014, The Linux Foundation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *     * Neither the name of The Linux Foundation, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.android.phone;

import android.content.Intent;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceScreen;
import android.telephony.SubscriptionInfo;
import android.telephony.SubscriptionManager;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.MenuItem;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;
import android.app.TabActivity;
import android.app.ActionBar;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.graphics.Color;

import java.util.List;

import static com.android.internal.telephony.PhoneConstants.SUBSCRIPTION_KEY;

public class SelectSubscription extends  TabActivity {

    private static final String LOG_TAG = "SelectSubscription";
    private static final boolean DBG = (PhoneGlobals.DBG_LEVEL >= 2);

    public static final String PACKAGE = "PACKAGE";
    public static final String TARGET_CLASS = "TARGET_CLASS";

    private String[] tabLabel = {"CARD 1", "CARD 2", "CARD 3"};

    private TabSpec subscriptionPref;

    @Override
    public void onPause() {
        super.onPause();
    }

    private static boolean hasCard(int index) {
        return TelephonyManager.getDefault().hasIccCard(index);
    }

    private static void setEnabled(View view, boolean enabled) {
        if (view == null) {
            return;
        }
        view.setEnabled(enabled);
        if (view instanceof TextView) {
            TextView text = (TextView) view;
            if (enabled) {
                text.setTextColor(Color.BLACK);
            } else {
                text.setTextColor(Color.GRAY);
            }
        } else if (view instanceof ViewGroup) {
            ViewGroup group = (ViewGroup) view;
            int count = group.getChildCount();
            for (int i = 0; i < count; i++) {
                setEnabled(group.getChildAt(i), enabled);
            }
        }
    }

    /*
     * Activity class methods
     */
    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        if (DBG) log("Creating activity");

        Intent intent =  getIntent();
        String pkg = intent.getStringExtra(PACKAGE);
        String targetClass = intent.getStringExtra(TARGET_CLASS);
        if (targetClass.equals("com.android.phone.MSimMobileNetworkSubSettings")){
           setTheme(R.style.MobileNetworkSettingsLight);
           setTitle(getString(R.string.network_settings_sel_sub));
        } else{
           setTheme(R.style.SettingsLight);
           setTitle(getString(R.string.callind_multi_sim));
        }

        setContentView(R.layout.multi_sim_setting);
        TabHost tabHost = getTabHost();


        int numPhones = TelephonyManager.getDefault().getPhoneCount();

        SubscriptionManager subscriptionManager =
                       SubscriptionManager.from(super.getApplicationContext());
        int tabIndex = -1;
        for (int i = 0; i < numPhones; i++) {
            SubscriptionInfo sir =
                    subscriptionManager.getActiveSubscriptionInfoForSimSlotIndex(i);
            String displayName =
                    ((sir != null)) ?
                    sir.getDisplayName().toString() : tabLabel[i];

            log("Creating SelectSub activity = " + i + " displayName = " + displayName);

            // Add phone Id to the displayName to differentiate
            // tag names of tabs
            subscriptionPref = tabHost.newTabSpec(displayName + i);
            subscriptionPref.setIndicator(displayName);
            intent = new Intent().setClassName(pkg, targetClass)
                    .setAction(intent.getAction());

            // For now using phone Id instead of subId
            SubscriptionManager.putPhoneIdAndSubIdExtra(intent, i);

            subscriptionPref.setContent(intent);
            tabHost.addTab(subscriptionPref);
            if (!hasCard(i)) {
                tabHost.setCurrentTab(i);
                setEnabled(tabHost.getCurrentTabView(), false);
                setEnabled(tabHost.getCurrentView(), false);
            } else {
                if (tabIndex < 0) {
                    tabIndex = i;
                }
            }
        }
        tabHost.setCurrentTab(tabIndex < 0 ? 0 : tabIndex);

        ActionBar actionBar = getActionBar();
        if (actionBar != null) {
            // android.R.id.home will be triggered in onOptionsItemSelected()
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    private static void log(String msg) {
        Log.d(LOG_TAG, msg);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        final int itemId = item.getItemId();
        if (itemId == android.R.id.home) {  // See ActionBar#setDisplayHomeAsUpEnabled()
             finish();
             return true;
        }
        return super.onOptionsItemSelected(item);
    }
}
